#include "stdafx.h"
#include "ParseTree.h"

#include "parser.h"
#include "lexer.h"
#include "conversion.h"

CParseTree::CParseTree()
{
}


CParseTree::~CParseTree()
{
}

void CParseTree::SayHello(char* const msg) const
{
	std::cout << "Msg from CParseTree: " << msg << std::endl;
}

void CParseTree::PutResult(double rlt)
{
	this->m_result = rlt;
}

void CParseTree::PutErrorMsg(const char* msg)
{
	std::ostringstream os;
	os << "At row - " << m_cell_index << " - " << msg;
	this->m_msgvctr.emplace_back(CharToWString(os.str().c_str()));
}

void CParseTree::PutErrorMsg(const wchar_t* msg)
{
	this->m_msgvctr.emplace_back(msg);
}

int CParseTree::CalculateCell(int index, const std::wstring& code)
{
	try
	{
		this->m_cell_index = index;

		yyscan_t scanner;
		yylex_init_extra(this, &scanner);

		size_t size = code.size();

		// allocate 1 more character (two more bytes)
		// for L'\0'
		wchar_t *buffer = new wchar_t[size + 1];

		wcscpy_s(buffer, size + 1, code.c_str());

		yy_scan_buffer((char*)buffer, (size + 1) * sizeof(wchar_t), scanner);

		int rlt = yyparse(scanner, this);

		yylex_destroy(scanner);
		
		delete[] buffer;

		// return value of yyparse()
		return rlt;

	}
	catch (std::exception&)
	{
		// let's say -1 signifies failure
		return -1;
	}
}

int CParseTree::RunParser(const std::string& filename)
{
	int rlt;

	try
	{
		yyscan_t scanner;

		FILE *in = nullptr;  errno_t er;

		if (filename != "")
			er = fopen_s(&in, filename.c_str(), "r");
		else
			in = stdin;

		yylex_init_extra(this, &scanner);
		yyset_in(in, scanner);

		rlt = yyparse(scanner, this);

		yylex_destroy(scanner);

		if (in != nullptr)
		{
			fclose(in);
			in = nullptr;
		}
	}
	catch (std::exception&)
	{

	}

	return rlt;
}


void CParseTree::copy_cstr(char** pTarget, const char* pSource)
{
	size_t size = strlen(pSource) + 1;
	*pTarget = new char[size];
	strcpy_s(*pTarget, size, pSource);
}

void CParseTree::copy_int(int& Target, const char* pSource)
{
	Target = atoi(pSource);
}
