#include "stdafx.h"

// in_SrcLen - byte count of in_Src
// in_MaxLen - character count of out_Dst
// out_Dst - should be allocated in advance
// converts multibyte character to wide character string
// mbctowc
BOOL AnsiToUnicode16L(CHAR *in_Src, INT in_SrcLen, WCHAR *out_Dst, INT in_MaxLen)
{
	/* locals */
	INT lv_Len;

	// do NOT decrease maxlen for the eos
	if (in_MaxLen <= 0)    return FALSE;

	// let windows find out the meaning of ansi  
	// - the SrcLen=-1 triggers MBTWC to add a eos to Dst and fails
	// if MaxLen is too small.  
	// - if SrcLen is specified then no eos is added  
	// - if (SrcLen+1) is specified then the eos IS added

	lv_Len = MultiByteToWideChar(CP_ACP, 0, in_Src, in_SrcLen, out_Dst, in_MaxLen);

	// validate
	if (lv_Len < 0)    lv_Len = 0;

	// ensure eos, watch out for a full buffersize  
	// - if the buffer is full without an eos then clear the output like MBTWC does  
	//   in case of too small outputbuffer  
	// - unfortunately there is no way to let MBTWC return shortened strings,  
	//   if the outputbuffer is too small then it fails completely

	if (lv_Len < in_MaxLen)    out_Dst[lv_Len] = 0;
	else if (out_Dst[in_MaxLen - 1])    out_Dst[0] = 0;

	// done
	return TRUE;
}

// this code is not optimized
// I just winged it.
CString CharToCString(const char* cstr)
{
	int in_SrcLen = (int)strlen(cstr);
	int in_MaxLen = (int)(in_SrcLen + 2);

	wchar_t *pBuffer = new wchar_t[in_MaxLen];

	if (AnsiToUnicode16L(const_cast<char*>(cstr), in_SrcLen, pBuffer, in_MaxLen))
	{
		CString rlt(pBuffer);

		delete[] pBuffer; return rlt;
	}

	delete[] pBuffer; return CString(L"");
}

std::wstring CharToWString(const char* cstr)
{
	int in_SrcLen = (int)strlen(cstr);
	int in_MaxLen = in_SrcLen + 2;

	wchar_t *pBuffer = new wchar_t[in_MaxLen];

	if (AnsiToUnicode16L(const_cast<char*>(cstr), in_SrcLen, pBuffer, in_MaxLen))
	{
		std::wstring rlt(pBuffer);

		delete[] pBuffer; return rlt;
	}

	delete[] pBuffer; return std::wstring(L"");
}

