	// output file definitions
%option outfile="lexer.cpp" header-file="lexer.h"

	// 8 bit scanner for unicode processing
	// reentrant scanner
%option 8bit reentrant noyywrap

	// we are going to use stack frame for "include" feature 
%option stack

	//%option warn nodefault

	// to interface with bison for reentrant parser
	// and scanner
%option bison-bridge bison-locations

	// line number tracking
%option yylineno

	// with multiple flex files are used in the same project
	// to prevent name crashes
	//%option prefix="tk"

	// our own parse tree data type
%option extra-type="CParseTree*"

	// put this block at the top of the 
	// generated scanner

	// these options are for windows compatability
%option always-interactive
	// these options are for windows compatability
%option nounistd

%top{

	// this macro defintion is windows compatability
#define YY_NO_UNISTD_H

#if defined(_MSC_BUILD)
	#pragma warning( disable: 4005)
#endif

	#include <iostream>
	#include <cmath>
	#include <cstring>

	using namespace std;

	#include "parser.h"
}

ZERO			[0][\x00]		
DIGIT			[1-9][\x00]
DIGITZ			{ZERO}|{DIGIT}
DIGITS			{DIGITZ}+

ATSYMBOL		[@][\x00]
EQUAL			[=][\x00]
NEWLINE			[\n][\x00]
SINGLEQ			([\x27][\x00])|([\xB9\xBC\xC8][\x02])|([\x18\x19\x32][\x20])|([\x8C][\xA7])|([\x01][\x03])
BASIC_LATIN		[A-Za-z_][\x00]

LETTER			{SINGLEQ}|{BASIC_LATIN}
LETTERS			{LETTER}+({DIGITS}|{LETTER})*

POWER			[eE][\x00]  
PLUSMINUS		[+-][\x00]

PERIOD			[.][\x00]
COMMA			[,][\x00]

ANYCHAR			(.){2}

BUTDBLQ			([^"][^\x00])|([^"][\x00])|(["][^\x00])
ANYBLNKD		([^\n"][^\x00])|([^\n"][\x00])|([\n"][^\x00])
BLANK			([^\n"\\][^\x00])|([^\n"\\][\x00])|([\n"\\][^\x00])
ANYBLNK			([^\n'"][^\x00])|([^\n'"][\x00])|([\n'"][^\x00])

FLTINTG			{ZERO}|{DIGITZ}*
HEADINTG		{DIGIT}{DIGITZ}{0,2}
FMTTAIL			{COMMA}{DIGITZ}{3}
FMTINTG			{HEADINTG}{FMTTAIL}*

FLATFLOAT		{FLTINTG}({PERIOD}{DIGITS})?
FLTREAL			({FLATFLOAT})({POWER}{PLUSMINUS}{DIGITS})?

FMTFLOAT		({FMTINTG}|{ZERO})({PERIOD}{DIGITS})?
FMTREAL			({FMTFLOAT})({POWER}{PLUSMINUS}{DIGITS})?

INTEGER			{FLTINTG}|{FMTINTG}
REAL			{FLTREAL}|{FMTREAL}
BUTNL			([^\x0A][^\x00])|([^\x0A][\x00])|([\x0A][^\x00])

%%

[+*/=();-][\x00]	{ return yytext[0]; }

{INTEGER}|{REAL}		{ yylval->dble = atof(yytext); return TKN_DBLE; }

{NEWLINE}|{BUTNL}		/* ignore all other characters */

%%
